%{
/* initial C code */
  extern char *yytext;

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include "ll.h"
#include "menu.h"
#include "options.h"
#include "pixmap.h"
#include "globals.h"
extern FILE *yyin;

FILE *yysetinput(const char *pfile);
void yycleanup();
int yylex();
int yyerror(const char *pstr);

static LPCONFIGLIST pConfigList;
static LPCONFIGLIST pCurrentList;
static LPCONFIGITEM pCurrentItem;

%}

/* decls */
%token STRING

/* grammar */
%%

main: at_least_one_expr
      ; 

at_least_one_expr: expr expr_list
                   ;

expr_list: /* blank */
          | expr expr_list
	  ;

expr: STRING {
               /* add the item */
               LPCONFIGITEM newItem;
               newItem = (LPCONFIGITEM)ListCreateNode((LPLIST)pCurrentList,sizeof(CONFIGITEM));
               newItem->name = strdup(yytext);
               newItem->value = 0;
               newItem->pSublist = 0;
               /* now add to the list */
               ListAddTail((LPLIST)pCurrentList,(LPNODE)newItem);
               /* set current item */
               pCurrentItem = newItem;
              } 
      '=' value ';' 
      ;

value: STRING       { /*finish*/ pCurrentItem->value = strdup(yytext);}
     | '{' { /* new sub list item */
             pCurrentItem->pSublist = CreateConfigItemList();
             pCurrentItem->pSublist->pParent = pCurrentList;
             pCurrentList = pCurrentItem->pSublist;
           }
        at_least_one_expr 
        '}' { /* restore the old current list */ 
              pCurrentList = pCurrentList->pParent;
            }
;


%%

/* ending C code */
int yyerror(const char *pstr)
{
  printf("Error: %s\n",pstr);
  return 0;
}

LPCONFIGLIST parse(const char *pFile)
{
  pConfigList = NULL;
  if (yysetinput(pFile))
  {
    pConfigList = CreateConfigItemList();
    pCurrentList = pConfigList;
    pCurrentItem = NULL;
    if (yyparse())
    {
      ListDestroy((LPLIST)pConfigList);
      pConfigList = NULL;
    }
    yycleanup();
  }
  return pConfigList;
}
